import json
import exifread
import sys
import os
import base64
import functools, operator

def findImageFiles(top):
    files = [[os.path.join(dn, f) for f in fl] for dn, _, fl in os.walk(top)]

    #flatten
    files = functools.reduce(operator.add, files)

    # filter
    files = filter(lambda name: os.path.splitext(name)[1] in ('.jpg', '.tiff'), files)

    return files

def readExif(filePath):
    # print(filePath)
    with open(filePath, 'rb') as f:
        try:
            tags = exifread.process_file(f, strict=True)
            return toJsonSerializable(tags)
        except ValueError:
            return {'__error__':True}

def toJsonSerializable(obj):
    if type(obj) in [int, float, str]:
        return obj
    elif type(obj) is bytes:
        return base64.b64encode(obj).decode()
    elif type(obj) is list:
        return [toJsonSerializable(v) for v in obj]
    elif type(obj) is dict:
        return {k:toJsonSerializable(v) for k,v in obj.items()}
    elif type(obj) is exifread.classes.Ratio:
        return str(obj)
    elif type(obj) is exifread.classes.IfdTag:
        typeName = exifread.FIELD_TYPES[obj.field_type][2]

        if typeName in ['Byte', 'Undefined'] and type(obj.values) is list:
            values = toJsonSerializable(bytes(obj.values))
        else:
            values = toJsonSerializable(obj.values)

        return {
            'tag': obj.tag,
            'field_type_value': obj.field_type,
            'field_type': typeName,
            'values': values,
            'printable': obj.printable,
            }
    else:
        return f'{obj} ({type(obj)})'

if __name__ == '__main__':
    if len(sys.argv) != 2:
        print('usage: build-data path/to/exif-samples', file=sys.stderr)
        sys.exit(1)

    pathToDir = sys.argv[1]
    inputFiles = findImageFiles(pathToDir)

    # files = {os.path.relpath(f, pathToDir): readExif(f) for f in inputFiles}
    files = {}
    for filePath in inputFiles:
        with open(filePath, 'rb') as f:
            try:
                tags = exifread.process_file(f, strict=True)
                if not tags:
                    files[filePath] = 'empty'
            except ValueError:
                files[filePath] = 'error'

    files = {'exif-samples-master/' + os.path.relpath(f, pathToDir): v for f,v in files.items()}

    print(f'{len(files)} files')

    with open('samples-exif.json', 'wb') as f:
        data = json.dumps(files, indent='  ').encode('utf-8')
        print(f'{len(data)} bytes')
        f.write(data)
